<?php
/**
 * @file
 * Taxonomy Term List bean plugin.
 */

class TaxListingBean extends BeanPlugin {

  /**
   * Declares default block settings.
   */
  public function values() {
    return array(
      'filters' => array(
        'vocabulary' => FALSE,
      ),
      'settings' => array(
        'term_view_mode' => 'full',
        'records_shown' => 5,
      ),
    );
  }

  /**
   * Builds extra settings for the block edit form.
   */
  public function form($bean, $form, &$form_state) {

    // Here we are defining the form.
    $form = array();

    // Select objects for the vocabularies that will be used.
    $vocabulary = taxonomy_get_vocabularies();
    $select_vocabulary_array = array();
    foreach ($vocabulary as $item) {
      $key = $item->vid;
      $value = $item->name;
      $select_vocabulary_array[$key] = $value;
    }

    // Building the basic form for individual 'related' bean configuration.
    $form['filters'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#title' => t('Filters'),
    );

    // Determine related taxonomy term vocabularies.
    $form['filters']['vocabulary'] = array(
      '#type' => 'select',
      '#title' => t('Vocabulary'),
      '#options' => $select_vocabulary_array,
      '#default_value' => $bean->filters['vocabulary'],
      '#required' => TRUE,
      '#multiple' => TRUE,
      '#size' => 5,
    );

    // Settings fieldset is used for configuring the 'listing' bean output.
    $form['settings'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#title' => t('Output'),
    );

    // Select objects for the taxonomy term view modes that will be used.
    $term_view_modes = array();
    $entity_info = entity_get_info();
    foreach($entity_info['taxonomy_term']['view modes'] as $key => $value) {
      $term_view_modes[$key] = $value['label'];
    }

    // User select which view mode to use for the term listing inside the bean.
    $form['settings']['term_view_mode'] = array(
      '#type' => 'select',
      '#title' => t('Taxonomy Term View Mode'),
      '#options' => $term_view_modes,
      '#default_value' => $bean->settings['term_view_mode'],
      '#required' => TRUE,
      '#multiple' => FALSE,
    );

    // User define a maximum number of terms to be shown.
    $form['settings']['records_shown'] = array(
      '#type' => 'textfield',
      '#title' => t('Records shown'),
      '#size' => 5,
      '#default_value' => $bean->settings['records_shown'],
      '#element_validate' => array('bean_tax_setting_is_numeric'),
    );

    return $form;
  }

  /**
   * Displays the bean.
   */
  public function view($bean, $content, $view_mode = 'default', $langcode = NULL) {

    // We need to make sure that the bean is configured correctly.
    if (!empty($bean->filters['vocabulary'])) {

      // Define an array of all taxonomy terms in the defined vocabularies.
      $possible_tid = array();
      foreach ($bean->filters['vocabulary'] as $vid) {
        $tree = taxonomy_get_tree($vid);
        foreach ($tree as $term) {
          array_push($possible_tid, $term->tid);
        }
      }

      // Compare possible terms to those attached to the node.
      if ($node = menu_get_object()) {
        $active_nid = $node->nid;
        $node_terms = bean_tax_get_node_terms($node);
        $valid_tid = array();
        foreach ($node_terms as $term) {
          if (in_array($term->tid, $possible_tid)) {
            array_push($valid_tid, $term->tid);
          }
        }

        if (empty($valid_tid)) {
          // There are no valid terms. Set Empty array for theme output.
          $content['terms'] = array();
        }
        else {
          // There are valid term results so we process further.
          $terms = taxonomy_term_load_multiple($valid_tid);
          $i = 1;
          $count = 0;
          // Invoke entity_view on the valid referenced terms inside 'records_shown' limit.
          foreach ($terms as $listing) {
            if ($i <= $bean->settings['records_shown']) {
              $content['terms'][$listing->tid] = entity_view('taxonomy_term', array($listing->tid => $listing), $bean->settings['term_view_mode']);
              $count++;
            }
            $i++;
          }
          // No relative nodes to be shown - show user friendly output.
          if ($count == 0) {
            $content['#markup'] = t('No Results');
          }
        }
      }
    return $content;
    }
  }
}

